from lisa_plugins_shopping.models import ShoppingList
from rest_framework import serializers


class ShoppingListSerializer(serializers.HyperlinkedModelSerializer):
    class Meta:
        model = ShoppingList
        fields = ('url', 'name', 'list')
        extra_kwargs = {
            'url': {'lookup_field': 'name'}
        }


class ItemSerializer(serializers.Serializer):
    items = serializers.ListField(child=serializers.CharField())



from lisa_api.lisa.logger import logger
from lisa_api.lisa.plugin import PluginBase


class ShoppingPlugin(PluginBase):
    def __init__(self):
        pass

    def add_intents(self):
        from lisa_api.api.models import Intent

        obj, created = Intent.objects.update_or_create(
            name='shopping_item_add',
            defaults={
                'method': 'POST',
                'api_url': '/api/v1/plugin-shopping/lists/{list_name}/item_add/'
            }
        )
        logger.debug("Adding {intent_name} intent for shopping plugin".format(intent_name=obj.name))

        obj, created = Intent.objects.update_or_create(
            name='shopping_item_delete',
            defaults={
                'method': 'POST',
                'api_url': '/api/v1/plugin-shopping/lists/{list_name}/item_delete/'
            }
        )
        logger.debug("Adding {intent_name} intent for shopping plugin".format(intent_name=obj.name))

        obj, created = Intent.objects.update_or_create(
            name='shopping_item_list',
            defaults={
                'method': 'GET',
                'api_url': '/api/v1/plugin-shopping/lists/{list_name}/item_list/'
            }
        )
        logger.debug("Adding {intent_name} intent for shopping plugin".format(intent_name=obj.name))

        obj, created = Intent.objects.update_or_create(
            name='shopping_list_add',
            defaults={
                'method': 'POST',
                'api_url': '/api/v1/plugin-shopping/lists/'
            }
        )
        logger.debug("Adding {intent_name} intent for shopping plugin".format(intent_name=obj.name))

        obj, created = Intent.objects.update_or_create(
            name='shopping_list_delete',
            defaults={
                'method': 'DELETE',
                'api_url': '/api/v1/plugin-shopping/lists/'
            }
        )
        logger.debug("Adding {intent_name} intent for shopping plugin".format(intent_name=obj.name))

        obj, created = Intent.objects.update_or_create(
            name='shopping_list_list',
            defaults={
                'method': 'GET',
                'api_url': '/api/v1/plugin-shopping/lists/'
            }
        )
        logger.debug("Adding {intent_name} intent for shopping plugin".format(intent_name=obj.name))

__title__ = 'Lisa Plugins Shopping'
__version__ = '0.1.12'
__author__ = 'Julien Syx'
__license__ = 'Apache'
__copyright__ = 'Copyright 2015 Julien Syx'

# Version synonym
VERSION = __version__

# Header encoding (see RFC5987)
HTTP_HEADER_ENCODING = 'iso-8859-1'

# Default datetime input and output formats
ISO_8601 = 'iso-8601'



# -*- coding: utf-8 -*-
from __future__ import unicode_literals

from django.db import models, migrations
import jsonfield.fields


class Migration(migrations.Migration):

    dependencies = [
    ]

    operations = [
        migrations.CreateModel(
            name='ShoppingList',
            fields=[
                ('id', models.AutoField(verbose_name='ID', serialize=False, auto_created=True, primary_key=True)),
                ('name', models.CharField(unique=True, max_length=100)),
                ('list', jsonfield.fields.JSONField()),
            ],
        ),
    ]



from lisa_plugins_shopping.views import ShoppingListViewSet
from rest_framework import routers
from django.conf.urls import include, url

router = routers.DefaultRouter()
router.register(r'lists', ShoppingListViewSet)

urlpatterns = [
    url(r'^/', include(router.urls)),
]



from django.db import models
from jsonfield import JSONField


class ShoppingList(models.Model):
    name = models.CharField(max_length=100, unique=True)
    list = JSONField()

    def save(self, force_insert=False, force_update=False, using=None,
             update_fields=None):
        self.name = self.name.lower()
        super(ShoppingList, self).save(force_insert, force_update)



#!/usr/bin/env python
# -*- coding: utf-8 -*-

from setuptools import setup
from setuptools.command.test import test as TestCommand
import re
import os
import sys


def get_version(package):
    """
    Return package version as listed in `__version__` in `init.py`.
    """
    init_py = open(os.path.join(package, '__init__.py')).read()
    return re.search("__version__ = ['\"]([^'\"]+)['\"]", init_py).group(1)


def get_packages(package):
    """
    Return root package and all sub-packages.
    """
    return [dirpath
            for dirpath, dirnames, filenames in os.walk(package)
            if os.path.exists(os.path.join(dirpath, '__init__.py'))]


def get_package_data(package):
    """
    Return all files under the root package, that are not in a
    package themselves.
    """
    walk = [(dirpath.replace(package + os.sep, '', 1), filenames)
            for dirpath, dirnames, filenames in os.walk(package)
            if not os.path.exists(os.path.join(dirpath, '__init__.py'))]

    filepaths = []
    for base, filenames in walk:
        filepaths.extend([os.path.join(base, filename)
                          for filename in filenames])
    return {package: filepaths}


version = get_version('lisa_plugins_shopping')


if sys.argv[-1] == 'publish':
    if os.system("pip freeze | grep wheel"):
        print("wheel not installed.\nUse `pip install wheel`.\nExiting.")
        sys.exit()
    if os.system("pip freeze | grep twine"):
        print("twine not installed.\nUse `pip install twine`.\nExiting.")
        sys.exit()
    os.system("python setup.py sdist bdist_wheel")
    os.system("twine upload dist/*")
    print("You probably want to also tag the version now:")
    print("  git tag -a %s -m 'version %s'" % (version, version))
    print("  git push --tags")
    sys.exit()


setup(
    name='lisa-plugins-shopping',
    version=version,
    url='https://github.com/project-lisa/lisa-plugins-shopping',
    license='Apache',
    description='LISA Plugins Shopping',
    author='Julien Syx',
    author_email='julien@lisa-project.net',  # SEE NOTE BELOW (*)
    packages=get_packages('lisa_plugins_shopping'),
    package_data=get_package_data('lisa_plugins_shopping'),
    install_requires=[
        'djangorestframework',
        'django',
        'lisa-api',
        'jsonfield'
    ],
    zip_safe=False,
    classifiers=[
        'Development Status :: 2 - Pre-Alpha',
        'Environment :: Web Environment',
        'Framework :: Django',
        'Intended Audience :: Developers',
        'License :: OSI Approved :: BSD License',
        'Operating System :: OS Independent',
        'Natural Language :: English',
        'Programming Language :: Python :: 2',
        'Programming Language :: Python :: 2.7',
        'Programming Language :: Python :: 3',
        'Programming Language :: Python :: 3.4',
        'Topic :: Internet :: WWW/HTTP',
    ],
    entry_points={
        'lisa.api.plugins': [
            'shopping = lisa_plugins_shopping:ShoppingPlugin',
        ]
    },
)

# (*) Please direct queries to the discussion group, rather than to me directly
#     Doing so helps ensure your question is helpful to other users.
#     Queries directly to my email are likely to receive a canned response.
#
#     Many thanks for your understanding.



from lisa_plugins_shopping.models import ShoppingList
from rest_framework import viewsets, status
from rest_framework.response import Response
from lisa_plugins_shopping.serializers import ShoppingListSerializer, ItemSerializer
from rest_framework.decorators import detail_route
from lisa_api.lisa.logger import logger
from django.utils.translation import ugettext as _


class ShoppingListViewSet(viewsets.ModelViewSet):
    """
    API endpoint that allows users to add/edit/delete shopping lists.
    """
    queryset = ShoppingList.objects.all()
    serializer_class = ShoppingListSerializer
    lookup_field = 'name'

    @detail_route(methods=['POST'], serializer_class=ItemSerializer)
    def item_delete(self, request, name=None):
        """
        This function manage the jsonfield of a shopping list to delete an item
        :param request:
        :param name:
        :return:

        Example (if the shopping list name is 'default'):
        curl -X POST -H "Content-Type: application/json" http://127.0.0.1:8000/api/v1/plugin-shopping/lists/default/item_delete/ --data '{"items": ["carotte", "chocolat"]}
        ---
        request_serializer: ItemSerializer
        response_serializer: ItemSerializer
        """
        list = self.get_object()
        json_list = list.list

        if not json_list.get('items'):
            json_list = {'items': []}

        if request.method == 'POST':
            serializer = ItemSerializer(data=request.data)
            if serializer.is_valid():
                for item in serializer.data['items']:
                    if item in json_list['items']:
                        json_list['items'].remove(item)
                        logger.debug('Removing item {item} of the list'.format(item=item))
                    else:
                        logger.debug('Item {item} does not exist'.format(item=item))
                list.list = json_list
                list.save()
                return Response(_('Item {items} has been removed from the list').format(
                    items=', '.join(serializer.data['items'])), status=status.HTTP_202_ACCEPTED)
            else:
                return Response(serializer.errors, status=status.HTTP_400_BAD_REQUEST)

    @detail_route(methods=['POST'], serializer_class=ItemSerializer)
    def item_add(self, request, name=None):
        """
        This function manage the jsonfield of a shopping list by adding a product
        :param request:
        :param name:
        :return:

        Example (if the shopping list name is 'default'):
        curl -X POST -H "Content-Type: application/json" http://127.0.0.1:8000/api/v1/plugin-shopping/lists/default/item_add/ --data '{"items": ["carotte", "chocolat"]}
        ---
        request_serializer: ItemSerializer
        response_serializer: ItemSerializer
        """
        list = self.get_object()
        json_list = list.list

        if not json_list.get('items'):
            json_list = {'items': []}

        if request.method == 'POST':
            serializer = ItemSerializer(data=request.data)
            if serializer.is_valid():
                logger.debug(serializer.data['items'])
                for item in serializer.data['items']:
                    if item not in json_list['items']:
                        json_list['items'].append(item)
                        logger.debug('Adding item {item} to the list'.format(item=item))
                    else:
                        logger.debug('Item {item} already exist'.format(item=item))
                list.list = json_list
                list.save()
                return Response(_('Item {items} has been added to the list').format(
                    items=', '.join(serializer.data['items'])), status=status.HTTP_201_CREATED)
            else:
                return Response(serializer.errors, status=status.HTTP_400_BAD_REQUEST)

    @detail_route(methods=['GET'], serializer_class=ItemSerializer)
    def item_list(self, request, name=None):
        """
        This function manage the jsonfield of a shopping list by listing all products.
        :param request:
        :param name:
        :return:

        Example (if the shopping list name is 'default'):
        curl -X GET -H "Content-Type: application/json" http://127.0.0.1:8000/api/v1/plugin-shopping/lists/default/item_list/
        ---
        request_serializer: ItemSerializer
        response_serializer: ItemSerializer
        """
        list = self.get_object()

        if request.method == 'GET':
            if 'items' not in list.list:
                return Response(_('You have nothing on your list {list_name}').format(
                                list_name=name
                                ), status=status.HTTP_200_OK)
            else:
                return Response(_('You have {items} on your list {list_name}').format(
                                items=', '.join(list.list['items']),
                                list_name=name
                                ), status=status.HTTP_200_OK)
